import QtQuick 2.15
import QtQuick.Controls 2.15
import QtQuick.Window 2.15

Window {
    width: 1280
    height: 720
    visible: true
    title: qsTr("Hello World")

    property Item pendingImage
    property string currentSource

    function replaceWhenLoaded() {
        if (pendingImage.status === Image.Loading) {
            return;
        }

        pendingImage.statusChanged.disconnect(replaceWhenLoaded);
        view.replace(pendingImage, {}, StackView.Transition);

        pendingImage = null;
    }

    Component {
        id: image

        Rectangle {
            property alias source: innerImage.source
            property alias status: innerImage.status

            color: "green"

            Image {
                id: innerImage
                anchors.fill: parent
                asynchronous: true
                cache: false
                autoTransform: true
            }
            StackView.onRemoved: destroy()
        }

    }

    Rectangle {
        anchors.fill: parent
        color: "black"

        TapHandler {
            onTapped: {
                currentSource = currentSource === "dark.png" ? "light.png" : "dark.png"
                pendingImage = image.createObject(view, {
                    "source": currentSource,
                    "opacity": 1,
                    "width": view.width,
                    "height": view.height,
                });

                pendingImage.statusChanged.connect(replaceWhenLoaded);
                replaceWhenLoaded();
            }
        }

        StackView {
            id: view
            anchors.fill: parent

            readonly property var colorList: ["green", "red", "yellow"]

            replaceEnter: Transition {
                enabled: false
                OpacityAnimator {
                    id: replaceEnterOpacityAnimator
                    from: 0
                    to: 1
                    duration: 1000
                }
            }
            replaceExit: Transition{
                PauseAnimation {
                    duration: replaceEnterOpacityAnimator.duration
                }
            }
        }
    }
}
